/***************************************************************************
 *            Utils.h
 *
 *  Copyright  2008  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef UTILS_H
#define UTILS_H

#include <string>

#include <boost/cstdint.hpp> 
#include <boost/filesystem.hpp>

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"

#if defined(_WIN32) && defined(__MINGW32__)
#include "Windows.h"
#endif

namespace fs = boost::filesystem;

namespace Lince
{
	fs::path build_filename(const fs::path&, const std::string&);
	std::string build_filename_as_string(const fs::path&, const std::string&);
	
	fs::path get_base_directory();
	fs::path get_torrents_base_directory();
	fs::path get_resume_data_base_directory();
	
	int to_int(const std::string &);
	boost::int64_t to_int64(const std::string &);
	
	std::string size_to_string(libtorrent::size_type);
	std::string time_to_string(boost::int64_t);
	
	void get_http_torrent(const std::string &, const std::string &);
	
	void save_entry(const fs::path&, const libtorrent::entry&);
	bool load_entry(const fs::path&, libtorrent::entry&);
	
	bool str_has_prefix(const std::string&, const std::string&);
	bool str_has_suffix(const std::string&, const std::string&);
		
} // namespace 

#endif // UTILS_H
