//           GraphStatsTab.cc
//  Tue Aug  7 20:06:50 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA


#include "GraphStatsTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include <iostream>
#include <math.h>

GraphicArea::GraphicArea()
{
	resolutionY = 25;
	resolutionX = 1000;
	maxSpeed = 10;
	numValues = 0;
	modify_bg(Gtk::STATE_NORMAL, Gdk::Color("white"));
	number = half = total = 0;
}

GraphicArea::~GraphicArea()
{
}

bool GraphicArea::redraw(){
	Glib::RefPtr<Gdk::Window> win = get_window();
    if (win)
    {
        Gdk::Rectangle r(0, 0, get_allocation().get_width(),
                get_allocation().get_height());
        win->invalidate_rect(r, false);
    }
    return true;
}


bool GraphicArea::on_expose_event(GdkEventExpose* event)
{
	// This is where we draw on the window
	Glib::RefPtr<Gdk::Window> window = get_window();
	if(window){
		Gtk::Allocation allocation = get_allocation();
		
		const int width = allocation.get_width();
		const int height = allocation.get_height();
		double xc, yc, temp_y, temp_x;
		
		xc = width * 0.94; yc = height * 0.94;
		
		Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
		
		// clip to the area indicated by the expose event so that we only redraw
		// the portion of the window that needs to be redrawn
		cr->rectangle(event->area.x, event->area.y, event->area.width, event->area.height);
		cr->clip();
		
		
		// Paint gray lines.
		cr->set_line_width(0.75);
		cr->set_source_rgb(0.90, 0.90, 0.90);
		
		float inc_bar = 4.0;
		
		temp_y = inc_bar;
		while(temp_y < height){
			cr->move_to(0, temp_y);
			cr->line_to(width, temp_y);
			temp_y += inc_bar;
		}
		
		cr->stroke();
		
		cr->set_line_width(2.0);  
		cr->set_source_rgb(0.0, 0.0, 0.0);
		
		// Paint Square.
		cr->move_to(0, 0);
		cr->line_to(0, height);
		cr->line_to(width, height);
		cr->line_to(width, 0);
		cr->line_to(0, 0);
		
		cr->stroke();
		
		cr->set_line_width(1.0);
		cr->move_to(xc, height);
		cr->line_to(xc, 0);
		
		
		// Numbers
		cr->set_font_size(12);
		cr->set_line_width(0.75);
		
		int inc_y;
		
		inc_y =  (int) rint( maxSpeed / ((height / resolutionY) - 1) );
		temp_y = height; int number=0; char num[10];
		if (!inc_y) inc_y = 1;
		
		while (temp_y > 0){
			sprintf(num, "%i KB/S", number);
			cr->move_to(width * 0.945, temp_y);
			cr->show_text(num);
			cr->move_to(0, temp_y);
			cr->line_to(xc, temp_y);
			number += inc_y ; temp_y -= resolutionY;
		}
		
		double temp_val = ((height / resolutionY) - (int) (height / resolutionY));
		double numberR;
		
		if (temp_val)
			numberR = number - (inc_y - temp_val);
		else
			numberR = number;

		cr->stroke();
		
		
		// Speed
		int nValues = values.size();
		values_t::iterator i = values.begin();
		double inc_x = (xc / resolutionX);
		double speed = height - ((height / numberR) * (*i));
		
		temp_x = xc;
		cr->set_line_width(1.5);
		cr->set_source_rgb(0.22, 0.61, 1.0);
		cr->move_to(temp_x, speed);
		
		while (nValues){
			speed = height - ((height / numberR) * (*i));
			cr->line_to(temp_x, speed);
			temp_x -= inc_x;
			nValues--; i++;
		}
		
		temp_x += inc_x;
		
		cr->line_to(temp_x, height);
		cr->line_to(xc, height);
		cr->close_path();		
		cr->fill_preserve();
		
		cr->stroke();
		
		// Half
		cr->set_source_rgb(1.0, 0.0, 0.0);
		
		speed = height - ((height / numberR) * (half));
		cr->move_to(0, speed);
		cr->line_to(xc, speed);
		
		cr->stroke();
		
  }

  return true;
}

void GraphicArea::setMaxSpeed(float speed){
	bool maxSpeedIsHalf=false;
	// Speed is in bytes set it to KB
	speed /= 1024;
	
	//if (values.size() == resolutionX && values.back() == maxSpeed){
	if (numValues == resolutionX){
		float maxValue = 0;
		numValues = 0;
		
		for (values_t::iterator i = values.begin(); i!=values.end(); ++i){
			if((*i) > maxValue){
				maxValue = (*i);
				maxSpeed = (*i);
			}
		}
	}
	
	if ((speed > maxSpeed) || (half > maxSpeed)){
		if (half > speed){
			maxSpeed = half;
			maxSpeedIsHalf = true;
		}
		else
			maxSpeed = speed;
		
		// To show the half line add 10%
		if (maxSpeedIsHalf) maxSpeed += (0.1 * maxSpeed);
	}
}

void GraphicArea::addValue(float value){
	// Value is in bytes set it to KB
	value /= 1024;
	
	if (number == 1000000){
		total = half;
		number = 1;
	}
	
	values.push_front(value);
	numValues++;
	if (values.size() > (resolutionX + 1)){
		values.pop_back();
	}
	number++;
	half = (total + value) / number;
	total += value;
}

GraphStatsTab::GraphStatsTab():
	nameTab(_("Speed")),
	graphicsWindow(true, 5)
{
	
		downloadsFrame.set_shadow_type(Gtk::SHADOW_OUT);
		downloadsFrame.set_label(_("Download Speed"));
		downloadsFrame.add(dG);
	
		uploadsFrame.set_shadow_type(Gtk::SHADOW_OUT);
		uploadsFrame.set_label(_("Upload Speed"));
		uploadsFrame.add(uG);
		
		graphicsWindow.pack_start(downloadsFrame, true, true);
		graphicsWindow.pack_start(uploadsFrame, true, true);
}


GraphStatsTab::~GraphStatsTab(){
}
