/***************************************************************************
 *            GuiSetupManager.cc
 *
 *  Sun Dec 17 01:37:36 2006
 *  Copyright  2006  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "../config.h"

#include "GuiSetupManager.h"

using namespace Lince;

const std::string GuiSetupManager::settingTags[] =
{
	// Strings
	"DefaultFileDirectory", "DownloadTorrentsDirectory", "LoadTorrentsDirectory",
	"LastLoadDirectory", "LastSaveDirectory",
	"SENTRY",
	// Ints
	"ShowTorrentsTab", "ShowOptionsTab", "ShowStatisticsTab", "ShowLogTab",
	"ShowToolBar", "UseDefaultFileDirecory", "ConfirmDataElimination",
	"UseDownloadTorrentsDirectory", "UseRemoveOriginalTorrent",
	"UseRemoveTorrentWhenFinished", "UseLoadTorrentsDirectory",
	"AllowChangeLoadDirectory", "ShowGraphicStatistics", "StatisticsTimeUpdate",
	"UseLogTab", "LogLevel", "UseColoredLogOutput", "SaveLogFile",
	"LogAutoScroll", "LogSize", "UseBtjunkie", "UseIsohunt", "UsePiratebay",
	"UseMininova", "UseTorrentReactor", "MaximumNumberOfSearches",
	"FilterResultsWithoutSeeders", "ShowSearchTab", "UseNotifications",
	"NotifyFinishedTorrents", "NotifyStoppedTorrents", "NotifyStartedTorrents",
	"NotifyAddedTorrents", "NotifyRemovedTorrents", "NotifyQueuedTorrents",
	"NotifyUnqueuedTorrents", "NotifyShowFromSystray",
	"SENTRY",
	// Int64
	"SENTRY"
};


GuiSetupManager::GuiSetupManager()
{
	for(int i=0; i<SETTINGS_LAST; i++)
		isSet[i] = false;

	for(int i=0; i<INT_LAST-INT_FIRST; i++)
	{
		intDefaults[i] = 0;
		intSettings[i] = 0;
	}

	for (int i=0; i < INT64_LAST-INT64_FIRST; i++)
	{
		int64Defaults[i] = 0;
		int64Settings[i] = 0;
	}

	// Strings
	setDefault(DEFAULT_FILE_DIRECTORY, "");
	setDefault(DOWNLOAD_TORRENTS_DIRECTORY, "");
	setDefault(LOAD_TORRENTS_DIRECTORY, "");
	setDefault(LAST_LOAD_DIRECTORY, "");
	setDefault(LAST_SAVE_DIRECTORY, "");

	// Ints
	setDefault(SHOW_TORRENTSTAB, true);
	setDefault(SHOW_OPTIONSTAB, true);
	setDefault(SHOW_STATISTICSTAB, true);
	setDefault(SHOW_LOGTAB, false);
	setDefault(SHOW_TOOLBAR, true);
	setDefault(USE_DEFAULT_FILE_DIRECTORY, false);
	setDefault(CONFIRM_DATA_ELIMINATION, true);
	setDefault(USE_DOWNLOAD_TORRENTS_DIRECTORY, false);
	setDefault(USE_REMOVE_ORIGINAL_TORRENT, true);
	setDefault(USE_REMOVE_TORRENT_WHEN_FINISHED, true);
	setDefault(USE_LOAD_TORRENTS_DIRECTORY, false);
	setDefault(ALLOW_CHANGE_LOAD_DIRECTORY, false);
	setDefault(SHOW_GRAPHIC_STATISTICS, true);
	setDefault(STATISTICS_TIME_UPDATE, 3);
	setDefault(USE_LOG_TAB, true);
	setDefault(LOG_LEVEL, 4);
	setDefault(LOG_COLORED_OUTPUT, true);
	setDefault(SAVE_LOG_TO_FILE, false);
	setDefault(LOG_AUTO_SCROLL, true);
	setDefault(LOG_SIZE, 512);
	setDefault(USE_BTJUNKIE, true);
	setDefault(USE_ISOHUNT, true);
	setDefault(USE_PIRATEBAY, true);
	setDefault(USE_MININOVA, true);
	setDefault(USE_TORRENTREACTOR, true);
	setDefault(MAX_NUMBER_OF_SEARCHES, 0);
	setDefault(FILTER_RESULTS_WITHOUT_SEEDERS, false);
	setDefault(SHOW_SEARCHTAB, false);
	setDefault(USE_NOTIFICATIONS, true);
	setDefault(NOTIFY_FINISHED_TORRENTS, true);
	setDefault(NOTIFY_STOPPED_TORRENTS, false);
	setDefault(NOTIFY_STARTED_TORRENTS, false);
	setDefault(NOTIFY_ADDED_TORRENTS, true);
	setDefault(NOTIFY_REMOVED_TORRENTS, true);
	setDefault(NOTIFY_QUEUED_TORRENTS, false);
	setDefault(NOTIFY_UNQUEUED_TORRENTS, false);
	setDefault(NOTIFY_SHOW_FROM_SYSTRAY, false);

	// Int64

	// Load file
	config_file = build_filename_as_string(get_base_directory(), "gui.xml");
	load();
}

GuiSetupManager::~GuiSetupManager()
{
}

GuiSetupManager::GuiSetupManager(const GuiSetupManager & s)
{
	int i;

	config_file = s.config_file;

	for (i=0; i < SETTINGS_LAST; i++)
		isSet[i] = s.isSet[i];

	for (i=0; i < INT_LAST-INT_FIRST; i++)
	{
		intDefaults[i] = s.intDefaults[i];
		intSettings[i] = s.intSettings[i];
	}

	for (i=0; i < STR_LAST-STR_FIRST; i++)
	{
		strDefaults[i] = s.strDefaults[i];
		strSettings[i] = s.strSettings[i];
	}

	for (i=0; i < INT64_LAST-INT64_FIRST; i++)
	{
		int64Defaults[i] = s.int64Defaults[i];
		int64Settings[i] = s.int64Settings[i];
	}
}

GuiSetupManager & GuiSetupManager::operator=(const GuiSetupManager & s)
{
	int i;

	if ( this != &s)
	{
		config_file = s.config_file;

		for (i=0; i < SETTINGS_LAST; i++)
			isSet[i] = s.isSet[i];

		for (i=0; i < INT_LAST-INT_FIRST; i++)
		{
			intDefaults[i] = s.intDefaults[i];
			intSettings[i] = s.intSettings[i];
		}

		for (i=0; i < STR_LAST-STR_FIRST; i++)
		{
			strDefaults[i] = s.strDefaults[i];
			strSettings[i] = s.strSettings[i];
		}

		for (i=0; i < INT64_LAST-INT64_FIRST; i++)
		{
			int64Defaults[i] = s.int64Defaults[i];
			int64Settings[i] = s.int64Settings[i];
		}
	}
	return *this;
}

void GuiSetupManager::save(){
	writer.setFileName(config_file);
	writer.startDocument("Lince");
	writer.startElement("Settings");

	for(int i=0; i<INT_LAST-INT_FIRST; i++)
	{
		writer.startElement(settingTags[i+INT_FIRST]);
		writer.addAttribute("Type", "int");
		if (!isSet[i+INT_FIRST])
			writer.addElementData(intDefaults[i]);
		else
			writer.addElementData(intSettings[i]);
		writer.endElement();
	}

	for(int i=0; i<STR_LAST-STR_FIRST; i++)
	{
		writer.startElement(settingTags[i]);
		writer.addAttribute("type", "string");
		if (!isSet[i+STR_FIRST])
			writer.addElementData(strDefaults[i]);
		else
			writer.addElementData(strSettings[i]);
		writer.endElement();
	}

	for(int i=0; i<INT64_LAST-INT64_FIRST; i++)
	{
		writer.startElement(settingTags[i+INT64_FIRST]);
		writer.addAttribute("type", "int64");
		if (!isSet[i+INT64_FIRST])
			writer.addElementData(int64Defaults[i]);
		else
			writer.addElementData(int64Settings[i]);
		writer.endElement();
	}

	writer.endDocument();
}

void GuiSetupManager::load()
{
	reader.setFileName(config_file);

	if (reader.findChild("Settings"))
	{
		reader.goToNextElement();

		int i;

		// Hay que revisarlo dependiendo del orden de busqueda algunas claves
		// pueden quedarse sin valor.
		// TODO Con esta version se arreglan los problemas pero tiene un coste
		// bastante mayor

		for(i=INT_FIRST; i<INT_LAST; i++)
		{
			const Glib::ustring& attr = settingTags[i];

            if(reader.findChildFromBeginning(attr))
            	set(IntSetting(i), to_int(reader.getChildData()));

		}

		for(i=STR_FIRST; i<STR_LAST; i++)
		{
			const Glib::ustring& attr = settingTags[i];

			if(reader.findChildFromBeginning(attr))
            	set(StrSetting(i), reader.getChildData());
		}

		for(i=INT64_FIRST; i<INT64_LAST; i++)
		{
			const Glib::ustring& attr = settingTags[i];

			if(reader.findChildFromBeginning(attr))
				set(Int64Setting(i), to_int64(reader.getChildData()));
		}
	}
}
