//           MakeTorrent.cc
//  Tue Nov 13 18:26:43 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include <boost/filesystem/operations.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/fstream.hpp>

#include "libtorrent/entry.hpp"
#include "libtorrent/bencode.hpp"
#include "libtorrent/file.hpp"
#include "libtorrent/storage.hpp"
#include "libtorrent/hasher.hpp"
#include "libtorrent/file_pool.hpp"
#include "libtorrent/create_torrent.hpp"

#include "Lince/Utils.h"
#include "Utils.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include "MakeTorrent.h"

MakeTorrent::MakeTorrent():
	Gtk::Dialog(_("New Torrent File"), true, true),
	torrentWindowLabel(_("New Torrent File")),
	fileButton(_("Add file")),
	directoryButton(_("Add directory")),
	announceLabel(_("Announce Url:")),
	commentLabel(_("Comment:")),
	privateTorrentButton(_("Private Torrent")),
	progressBar()
{
	progressBar.set_text(_("Making Torrent"));
	
	cancelButton = add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	okButton = add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);
		
	// Signals
	okButton->signal_clicked().connect( sigc::mem_fun(this, &MakeTorrent::prepareToMakeTorrent) );
	cancelButton->signal_clicked().connect( sigc::mem_fun(this, &MakeTorrent::hideWindow) );
	
	// Dispatcher Signals
	finishTorrentFinishSignal.connect( sigc::mem_fun(this, &MakeTorrent::hideWindow) );
	updateProgressBarSignal.connect( sigc::mem_fun(this, &MakeTorrent::updateProgressBar) );
	
	fileButton.signal_clicked().connect( sigc::mem_fun(this, &MakeTorrent::addFile) );
	directoryButton.signal_clicked().connect( sigc::mem_fun(this, &MakeTorrent::addDirectory) );
	
	filesFrame.set_shadow_type(Gtk::SHADOW_OUT);
	filesFrame.set_label(_("Files"));
	
	refTagTable = Gtk::TextBuffer::TagTable::create();
	TextBuffer = Gtk::TextBuffer::create(refTagTable);
	
	filesTextView.set_buffer(TextBuffer);
	filesTextView.set_cursor_visible (false);
	filesTextView.set_editable(false);
	
	scrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	scrolledWindow.add(filesTextView);
	
	Gtk::Alignment * spacer = Gtk::manage(new Gtk::Alignment);
	
	filesHBox.pack_start(*spacer, Gtk::PACK_EXPAND_WIDGET , 5);
	filesHBox.pack_start(fileButton, Gtk::PACK_SHRINK , 5);
	filesHBox.pack_start(directoryButton, Gtk::PACK_SHRINK , 5);	
	
	filesVBox.pack_start(scrolledWindow, Gtk::PACK_EXPAND_WIDGET , 5);
	filesVBox.pack_start(filesHBox, Gtk::PACK_SHRINK , 5);
	
	filesFrame.add(filesVBox);
		
	Gtk::Table * table = Gtk::manage(new Gtk::Table(2,2));
	
	announceEntry.set_text("http://");
		
	announceLabel.property_xalign()  = 0.05;
	commentLabel.property_xalign()  = 0.05;
		
	table->attach(announceLabel, 0, 1, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	table->attach(announceEntry, 1, 2, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
		
	table->attach(commentLabel, 0, 1, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	table->attach(commentEntry, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
		
	Gtk::HSeparator *hBar = Gtk::manage(new Gtk::HSeparator);
	Gtk::HSeparator *hBar2 = Gtk::manage(new Gtk::HSeparator);
	
	Gtk::VBox *vbox = get_vbox();
		
	vbox->pack_start(torrentWindowLabel, Gtk::PACK_SHRINK, 0);
	vbox->pack_start(filesFrame, Gtk::PACK_EXPAND_WIDGET, 5);
	vbox->pack_start(*hBar, Gtk::PACK_SHRINK, 5);
	vbox->pack_start(*table, Gtk::PACK_SHRINK, 5);
	vbox->pack_start(privateTorrentButton, Gtk::PACK_SHRINK, 0);
	vbox->pack_start(*hBar2, Gtk::PACK_SHRINK, 5);
	vbox->pack_start(progressBar, Gtk::PACK_SHRINK, 5);
	
	offset = 0;
	progress = 0;
	pulse = 0;
		
	set_gravity(Gdk::GRAVITY_CENTER);
	show_all();
	progressBar.hide();
	hide();
}

void MakeTorrent::hideWindow(){
	progressBar.set_fraction(0);
	progressBar.hide();
	progress = pulse = 0;
	
	okButton->property_sensitive () = true;
	cancelButton->property_sensitive () = true;
	
	path = boost::filesystem::path();
	announceEntry.set_text("http://");
	commentEntry.set_text("");
	privateTorrentButton.set_active(false);	
							   
	Gtk::TextIter begin, end;
	TextBuffer->get_bounds(begin, end);
	TextBuffer->erase(begin, end);
	offset = 0;
	
	hide();
}

void MakeTorrent::prepareToMakeTorrent(){
	if (path.empty()){
		Utils::warningDialog(_("Making Torrent"), _("There are no files added"));
		return;
	}
	
	Glib::ustring announceURL = announceEntry.get_text();
	if (!Utils::isAnURL(announceURL)){
		Utils::warningDialog(_("Making Torrent"), _("Announce entry is not a valid URL"));
		return;
	}
			
	Glib::ustring torrentFileName = Utils::getFilenameFromSaveDialog(_("Save Torrent File"), "", "");
		
	if (!torrentFileName.size())
		return;
	
	progressBar.show();
	okButton->property_sensitive () = false;
	cancelButton->property_sensitive () = false;
	
	Glib::Thread::create(sigc::bind<Glib::ustring> (sigc::mem_fun(this, &MakeTorrent::makeTorrent), torrentFileName), true);
}

void MakeTorrent::launchUpdateProgressBar(int)
{
	updateProgressBarSignal.emit();
}

void MakeTorrent::makeTorrent(Glib::ustring torrentFileName){
	libtorrent::file_storage fs;
	
	libtorrent::add_files(fs, path);
	
	libtorrent::create_torrent t(fs);
	
	t.add_tracker(announceEntry.get_text().c_str());
	t.set_creator(PACKAGE_NAME);
	t.set_comment(commentEntry.get_text().c_str());	
	t.set_priv(privateTorrentButton.get_active());
	
	pulse = double(1/(double) fs.num_pieces());
	
	libtorrent::set_piece_hashes(t, path.branch_path(), 
				boost::bind(&MakeTorrent::launchUpdateProgressBar, this, _1));
	
	Lince::save_entry(torrentFileName.c_str(), t.generate());
	
	finishTorrentFinishSignal.emit();
}

void MakeTorrent::updateProgressBar(){
	progressBar.set_fraction(progress);
	progress += pulse;
}

void MakeTorrent::addFile(){
	if (offset != 0) return;
	Glib::ustring file;
	file = Utils::getFilenameFromLoadDialog(_("Choose a file"), "", "", "");
	
	if (file == "") return;
	
	path = boost::filesystem::complete(file.c_str());
	addFiles(path.branch_path(), path.leaf());
}

void MakeTorrent::addDirectory(){
	if (offset != 0) return;
	Glib::ustring directory;
	directory = Utils::getDirectoryFromDialog(_("Choose a directory"));
	
	if (directory == "") return;
	
	path = boost::filesystem::complete(directory.c_str());
	addFiles(path.branch_path(), path.leaf());
}

void MakeTorrent::addFiles(boost::filesystem::path const& p, boost::filesystem::path const& l)
{
	if (l.leaf()[0] == '.') return;
	boost::filesystem::path f(p / l);
	
	if (boost::filesystem::is_directory(f)){
		for (boost::filesystem::directory_iterator i(f), end; i != end; ++i){
			try{
				addFiles(p, l / i->filename());
			}
			catch(std::exception)
			{
			}
		}
	}
	else{
		TextBuffer->insert(TextBuffer->get_iter_at_offset(offset), l.string());
		offset += l.string().size();
		TextBuffer->insert(TextBuffer->get_iter_at_offset(offset), std::string("\n"));
		offset += 1;
	}
}						   
						   
MakeTorrent::~MakeTorrent(){
}
