/***************************************************************************
 *            MenuBar.cc
 *
 *  Tue Jan 23 23:28:34 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "MenuBar.h"
#include "MainWindow.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include "Lince/Core.h"
#include "Lince/SessionManager.h"
#include "Lince/TorrentsManager.h"

using namespace Lince;

MenuBar::MenuBar(MainWindow* window){

	//Glib::RefPtr<Gtk::IconFactory> toolBarfactory = Gtk::IconFactory::create();
	//Utils::registerIcon(toolBarfactory, "globe.png", "openTorrent", "Open Torrent");
	//toolBarfactory->add_default();
	
	m_refActionGroup = Gtk::ActionGroup::create();
	m_refActionGroup->add( Gtk::Action::create("File", _("_File")) );
	m_refActionGroup->add( Gtk::Action::create("OpenTorrent", Gtk::Stock::OPEN, _("_Open Torrent"), _("Open Torrent")), sigc::mem_fun(window, &MainWindow::select_torrent_file_dialog) );
	//m_refActionGroup->add( Gtk::Action::create("OpenTorrent", Gtk::StockID("openTorrent"), _("_Open Torrent"), "Open Torrent"), sigc::mem_fun(window, &MainWindow::select_torrent_file_dialog) );
	m_refActionGroup->add( Gtk::Action::create("NewTorrent", _("_New Torrent"), "New Torrent"), sigc::mem_fun(window, &MainWindow::make_torrent) );
	m_refActionGroup->add( Gtk::Action::create("CloseTab", Gtk::Stock::CLOSE, _("_Close Tab"), "Close Tab"), sigc::mem_fun(window, &MainWindow::remove_current_tab) );
	m_refActionGroup->add( Gtk::Action::create("Quit", Gtk::Stock::QUIT, _("_Quit"), _("Quit")), sigc::mem_fun(window, &MainWindow::signal_exit) );
		
	// View menu
	m_refActionGroup->add( Gtk::Action::create("View", _("_View")) );
	m_refActionGroup->add( Gtk::Action::create("TorrentsTab", _("_Torrents"),  "Torrents"), sigc::mem_fun(window, &MainWindow::add_torrents_tab) );
	m_refActionGroup->add( Gtk::Action::create("OptionsTab", _("_Options"),  "Options"), sigc::mem_fun(window, &MainWindow::add_options_tab) );
	m_refActionGroup->add( Gtk::Action::create("StatisticsTab", _("_Statistics"),  "Statistics"), sigc::mem_fun(window, &MainWindow::add_statistics_tab) );
	m_refActionGroup->add( Gtk::Action::create("LogTab", _("_Log"),  "Log"), sigc::mem_fun(window, &MainWindow::add_log_tab) );
	m_refActionGroup->add( Gtk::Action::create("SearchTab", _("S_earch"),  "Search"), sigc::mem_fun(window, &MainWindow::add_search_tab) );
	showTBButton = Gtk::ToggleAction::create("ShowTBButton", _("Show tool bar"), "Show tool bar");
	m_refActionGroup->add( showTBButton, sigc::mem_fun(this, &MenuBar::manage_tool_bar));
	
	// Transference menu
	m_refActionGroup->add( Gtk::Action::create("Transference", _("_Transference")) );
	m_refActionGroup->add( Gtk::Action::create("ResumeAllTorrents", _("Resume all")), sigc::mem_fun(window, &MainWindow::start_all_torrents) );
	m_refActionGroup->add( Gtk::Action::create("StopAllTorrents", _("Stop all")), sigc::mem_fun(window, &MainWindow::stop_all_torrents) );
			
	// Torrent menu
	m_torrentMenu = Gtk::Action::create("Torrent", _("_Torrent"));
	m_refActionGroup->add( m_torrentMenu );
	m_torrentMenu->set_sensitive(false);
	m_refActionGroup->add( Gtk::Action::create("ShowDetails", Gtk::Stock::DIALOG_INFO, _("Show Details"), "Show Details"), sigc::mem_fun(window, &MainWindow::add_tinfo_tab_from_menu) );	
	
	m_refActionGroup->add( Gtk::Action::create("Speed", _("_Speed")) );
	
	m_refActionGroup->add( Gtk::Action::create("Upload", _("Upload")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedus", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), -1) );
	m_refActionGroup->add( Gtk::Action::create("20us", "20 KB", "20 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 20) );
	m_refActionGroup->add( Gtk::Action::create("40us", "40 KB", "40 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 40) );
	m_refActionGroup->add( Gtk::Action::create("60us", "60 KB", "60 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 60) );
	m_refActionGroup->add( Gtk::Action::create("80us", "80 KB", "80 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 80) );
	m_refActionGroup->add( Gtk::Action::create("100us", "100 KB", "100 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 100) );
	m_refActionGroup->add( Gtk::Action::create("200us", "200 KB", "200 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 200) );
	m_refActionGroup->add( Gtk::Action::create("400us", "400 KB", "400 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 400) );
	m_refActionGroup->add( Gtk::Action::create("Manualus", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_speed), 0) );
	
	m_refActionGroup->add( Gtk::Action::create("Download", _("Download")) );
	m_refActionGroup->add( Gtk::Action::create("Unlimitedds", _("Unlimited"), "Unlimited"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), -1) );
	m_refActionGroup->add( Gtk::Action::create("20ds", "20 KB", "20 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 20) );
	m_refActionGroup->add( Gtk::Action::create("40ds", "40 KB", "40 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 40) );
	m_refActionGroup->add( Gtk::Action::create("60ds", "60 KB", "60 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 60) );
	m_refActionGroup->add( Gtk::Action::create("80ds", "80 KB", "80 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 80) );
	m_refActionGroup->add( Gtk::Action::create("100ds", "100 KB", "100 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 100) );
	m_refActionGroup->add( Gtk::Action::create("200ds", "200 KB", "200 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 200) );
	m_refActionGroup->add( Gtk::Action::create("400ds", "400 KB", "400 KB"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 400) );
	m_refActionGroup->add( Gtk::Action::create("Manualds", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_download_speed), 0) );
	
	m_refActionGroup->add( Gtk::Action::create("Connections", _("_Connections")) );
	
	m_refActionGroup->add( Gtk::Action::create("MaxConnections", _("Max Connections")) );
	m_refActionGroup->add( Gtk::Action::create("Disabledmc", _("Disabled"), "_Disabled"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_max_connections), -1) );
	m_refActionGroup->add( Gtk::Action::create("10mc", "10", "10"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_max_connections), 10) );
	m_refActionGroup->add( Gtk::Action::create("20mc", "20", "20"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_max_connections), 20) );
	m_refActionGroup->add( Gtk::Action::create("40mc", "40", "40"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_max_connections), 40) );
	m_refActionGroup->add( Gtk::Action::create("80mc", "80", "80"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_max_connections), 80) );
	m_refActionGroup->add( Gtk::Action::create("100mc", "100", "100"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_max_connections), 100) );
	m_refActionGroup->add( Gtk::Action::create("Manualmc", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_max_connections), 0) );
	
	m_refActionGroup->add( Gtk::Action::create("UploadSlots", _("Upload Slots")) );
	m_refActionGroup->add( Gtk::Action::create("Disabledul", _("Disabled"), "_Disabled"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_slots), -1) );
	m_refActionGroup->add( Gtk::Action::create("3ul", "3", "3"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_slots), 3) );
	m_refActionGroup->add( Gtk::Action::create("6ul", "6", "6"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_slots), 6) );
	m_refActionGroup->add( Gtk::Action::create("12ul", "12", "12"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_slots), 12) );
	m_refActionGroup->add( Gtk::Action::create("24ul", "24", "24"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_slots), 24) );
	m_refActionGroup->add( Gtk::Action::create("Manualul", _("Manual"), "Manual"), sigc::bind<int> (sigc::mem_fun(window, &MainWindow::set_torrent_upload_slots), 0) );
	
	m_refActionGroup->add( Gtk::Action::create("ShareRatio", _("_Share Ratio")) );
	m_refActionGroup->add( Gtk::Action::create("Disabledsr", _("Disabled"), "_Disabled"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 0) );
	m_refActionGroup->add( Gtk::Action::create("1sr", "1", "1"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 1) );
	m_refActionGroup->add( Gtk::Action::create("1.5sr", "1.5", "1.5"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 1.5) );
	m_refActionGroup->add( Gtk::Action::create("2sr", "2", "2"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 2) );
	m_refActionGroup->add( Gtk::Action::create("2.5sr", "2.5", "2.5"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 2.5) );
	m_refActionGroup->add( Gtk::Action::create("3sr", "3", "3"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 3) );
	m_refActionGroup->add( Gtk::Action::create("3.5sr", "3.5", "3.5"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 3.5) );
	m_refActionGroup->add( Gtk::Action::create("4sr", "4", "4"), sigc::bind<double> (sigc::mem_fun(window, &MainWindow::set_torrent_share_ratio), 4) );
		
	m_refActionGroup->add( Gtk::Action::create("Tracker", _("Tracker")) );
	m_refActionGroup->add( Gtk::Action::create("EditTrackers", _("Edit trackers")), sigc::mem_fun(window, &MainWindow::edit_torrent_trackers) );
	m_refActionGroup->add( Gtk::Action::create("TrackerLoginData", _("Set Tracker log-in")), sigc::mem_fun(window, &MainWindow::trackerLogin) );
	m_refActionGroup->add( Gtk::Action::create("ForceReannounce", _("Force Reannounce")), sigc::mem_fun(window, &MainWindow::torrent_force_reannounce) );
	
	resume_torrent = Gtk::Action::create("ResumeTorrent", Gtk::Stock::MEDIA_PLAY, _("Resume"), _("Resume"));
	stop_torrent = Gtk::Action::create("StopTorrent", Gtk::Stock::MEDIA_STOP, _("Stop"), _("Stop"));
	
	m_refActionGroup->add( resume_torrent, sigc::mem_fun(window, &MainWindow::resume_torrents) );
	m_refActionGroup->add( stop_torrent, sigc::mem_fun(window, &MainWindow::stop_torrents) );

#if defined(_WIN32) && defined(__MINGW32__)
	m_refActionGroup->add( Gtk::Action::create("RemoveTorrent", Gtk::Stock::CANCEL, _("Remove"), "Remove"), sigc::mem_fun(window, &MainWindow::remove_torrent) );	
#else
	remove_torrent = Gtk::Action::create("RemoveTorrent", Gtk::Stock::DELETE, _("Remove"), _("Remove"));
	m_refActionGroup->add( remove_torrent, sigc::mem_fun(window, &MainWindow::remove_torrents) );	
#endif
	
	// Help menu
	m_refActionGroup->add( Gtk::Action::create("Help", _("_Help")) );
	m_refActionGroup->add( Gtk::Action::create("About",_("About"), "About"), sigc::mem_fun(*this, &MenuBar::showAboutDialog) );
	
	
	// Tool bar
	move_torrent_top = Gtk::Action::create("GoTop", Gtk::Stock::GOTO_TOP, _("Go Top"), _("Go Top"));
	move_torrent_up = Gtk::Action::create("GoUp", Gtk::Stock::GO_UP, _("Go Up"), _("Go Up"));
	move_torrent_down = Gtk::Action::create("GoDown", Gtk::Stock::GO_DOWN, _("Go Down"), _("Go Down"));
	move_torrent_bottom = Gtk::Action::create("GoBottom", Gtk::Stock::GOTO_BOTTOM, _("Go Bottom"), _("Go Bottom"));
	
	m_refActionGroup->add( move_torrent_top, sigc::mem_fun(window, &MainWindow::move_torrent_top) );	
	m_refActionGroup->add( move_torrent_up, sigc::mem_fun(window, &MainWindow::move_torrent_up) );	
	m_refActionGroup->add( move_torrent_down, sigc::mem_fun(window, &MainWindow::move_torrent_down) );	
	m_refActionGroup->add( move_torrent_bottom, sigc::mem_fun(window, &MainWindow::move_torrent_bottom) );
		
#if !(defined(_WIN32) && defined(__MINGW32__))
	m_refActionGroup->add( Gtk::Action::create("Test", Gtk::Stock::DIALOG_QUESTION, "Test", "Test"), sigc::mem_fun(window, &MainWindow::show_config_options) );
#endif
	
	
	m_refUIManager = Gtk::UIManager::create();
	m_refUIManager->insert_action_group(m_refActionGroup);
		
	window->add_accel_group(m_refUIManager->get_accel_group());
		 
	Glib::ustring ui_info =
		"<ui>"
		" <menubar name='MenuBar'>"
		"  <menu action='File'>"
		"  <menuitem action='OpenTorrent'/>"
		"  <menuitem action='NewTorrent'/>"
		"  <separator/>"
		"  <menuitem action='CloseTab'/>"
		"  <separator/>"
		"  <menuitem action='Quit'/>"
		"  </menu>"
		"  <menu action='View'>"
		"  <menuitem action='TorrentsTab'/>"
		"  <menuitem action='OptionsTab'/>"
		"  <menuitem action='StatisticsTab'/>"
		"  <menuitem action='LogTab'/>"
		"  <menuitem action='SearchTab'/>"
		"  <menuitem action='ShowTBButton'/>"
		"  </menu>"
		"  <menu action='Transference'>"
		"  <menuitem action='ResumeAllTorrents'/>"
		"  <menuitem action='StopAllTorrents'/>"
		"  </menu>"
		"  <menu action='Torrent'>"
		"  <menuitem action='ShowDetails'/>"
		"  <separator/>"
		"  <menu action='Speed'>"
			"	 <menu action='Upload'>"
			"    <menuitem action='Unlimitedus'/>"
			"    <separator/>"
			"    <menuitem action='20us'/>"
			"    <menuitem action='40us'/>"
			"    <menuitem action='60us'/>"
			"    <menuitem action='80us'/>"
			"    <menuitem action='100us'/>"
			"    <menuitem action='200us'/>"
			"    <menuitem action='400us'/>"
			"    <separator/>"
			"    <menuitem action='Manualus'/>"
			"	 </menu>"		
			"	 <menu action='Download'>"
			"    <menuitem action='Unlimitedds'/>"
			"    <separator/>"
			"    <menuitem action='20ds'/>"
			"    <menuitem action='40ds'/>"
			"    <menuitem action='60ds'/>"
			"    <menuitem action='80ds'/>"
			"    <menuitem action='100ds'/>"
			"    <menuitem action='200ds'/>"
			"    <menuitem action='400ds'/>"
			"    <separator/>"
			"    <menuitem action='Manualds'/>"
			"	 </menu>"
		"  </menu>"
		"	 <menu action='Connections'>"
		"	 <menu action='MaxConnections'>"
			"    <menuitem action='Disabledmc'/>"
			"    <separator/>"
			"    <menuitem action='10mc'/>"
			"    <menuitem action='20mc'/>"
			"    <menuitem action='40mc'/>"
			"    <menuitem action='80mc'/>"
			"    <menuitem action='100mc'/>"
			"    <separator/>"
			"    <menuitem action='Manualmc'/>"
		"	 </menu>"
		"	 <menu action='UploadSlots'>"
			"    <menuitem action='Disabledul'/>"
			"    <separator/>"
			"    <menuitem action='3ul'/>"
			"    <menuitem action='6ul'/>"
			"    <menuitem action='12ul'/>"
			"    <menuitem action='24ul'/>"
			"    <separator/>"
			"    <menuitem action='Manualul'/>"
			"	 </menu>"
		"	 </menu>"
		"	 <menu action='ShareRatio'>"
			"    <menuitem action='Disabledsr'/>"
			"    <separator/>"
			"    <menuitem action='1sr'/>"
			"    <menuitem action='1.5sr'/>"
			"    <menuitem action='2sr'/>"
			"    <menuitem action='2.5sr'/>"
			"    <menuitem action='3sr'/>"
			"    <menuitem action='3.5sr'/>"
			"    <menuitem action='4sr'/>"
		"	 </menu>"
		"	 <menu action='Tracker'>"
			"    <menuitem action='EditTrackers'/>"
			"    <menuitem action='TrackerLoginData'/>"
			"    <menuitem action='ForceReannounce'/>"
		"	 </menu>"
		"  <separator/>"
		"  <menuitem action='ResumeTorrent'/>"
		"  <menuitem action='StopTorrent'/>"
		"  <separator/>"
		"  <menuitem action='RemoveTorrent'/>"
		"  </menu>"	
		"  <menu action='Help'>"
		"  <menuitem action='About'/>"
		"  </menu>"	
		" </menubar>"
		" <toolbar  name='ToolBar'>"
		"  <toolitem action='OpenTorrent'/>"
		"  <separator/>"
		"  <toolitem action='ResumeTorrent'/>"
		"  <toolitem action='StopTorrent'/>"
		"  <separator/>"
		"  <toolitem action='RemoveTorrent'/>"
		"  <separator/>"
		"  <toolitem action='GoTop'/>"
		"  <toolitem action='GoUp'/>"
		"  <toolitem action='GoDown'/>"
		"  <toolitem action='GoBottom'/>"
		"  <separator/>"
		"  <toolitem action='Quit'/>"
		//"  <toolitem action='Test'/>"
		" </toolbar>"
		"</ui>";
	
	try{
		m_refUIManager->add_ui_from_string(ui_info);
	}
	catch(const Glib::Error& ex){
		std::cerr << "building menus failed: " <<  ex.what();
	}
}

void MenuBar::setShowToolBar(bool action)
{
	showTBButton->set_active(action);
}

void MenuBar::manage_tool_bar()
{
	if (showTBButton->get_active())
		showToolBar();
	else
		hideToolBar();
}

MenuBar::~MenuBar(){
}

void MenuBar::showAboutDialog(){
	Gtk::AboutDialog aboutDialog;
	std::list<Glib::ustring> authors;
	
	authors.push_back("Fernando Tarín Morales - icemanf@gmail.com");
	
	aboutDialog.set_authors(authors);
	aboutDialog.set_copyright("Copyright (C) 2009  Fernando Tarín Morales");
	aboutDialog.set_name(PACKAGE_NAME);
	aboutDialog.set_version(PACKAGE_VERSION);
	aboutDialog.set_translator_credits(_("translator-credits"));
	
	aboutDialog.set_website("http://lincetorrent.sourceforge.net/");
	aboutDialog.set_website_label("http://lincetorrent.sourceforge.net/");
	
	aboutDialog.set_license(Glib::ustring(_("This program is free software: you can redistribute it and/or modify\n")) +
							Glib::ustring(_("it under the terms of the GNU General Public License as published by\n")) +
							Glib::ustring(_("the Free Software Foundation, either version 3 of the License, or\n")) +
							Glib::ustring(_("(at your option) any later version.\n")) +
							Glib::ustring(_("This program is distributed in the hope that it will be useful,\n")) +
							Glib::ustring(_("but WITHOUT ANY WARRANTY; without even the implied warranty of\n")) +
							Glib::ustring(_("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n")) +
							Glib::ustring(_("GNU General Public License for more details.\n")) +
							Glib::ustring(_("You should have received a copy of the GNU General Public License\n")) +
							Glib::ustring(_("along with this program.  If not, see <http://www.gnu.org/licenses/>.")));
	
	aboutDialog.run();
}
