//           OptionsInfoNoteBookTab.cc
//  Mon Sep 10 00:55:27 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

 
#include "OptionsInfoNoteBookTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

using namespace Lince;

OptionsInfoNoteBookTab::OptionsInfoNoteBookTab():
	nameTab(_("Options")),
	maxUploadLabel(_("Maximum upload speed (in KB/s)")),
	maxUploadCheckButton(_("Disable")),
	maxDownloadLabel(_("Maximum download speed (in KB/s)")),
	maxDownloadCheckButton(_("Disable")),
	maxConnectsLabel(_("Maximum number of connections:")),
	maxConnectsCheckButton(_("Disable")),
	maxUploadSlotsLabel(_("Maximum number of upload slots:")),
	maxUploadSlotsCheckButton(_("Disable")),
	shareRatioLabel1(_("Share ratio:\t")),
	shareRatioLabel2("1 KB DL ="),
	shareRatioLabel3("KB UP"),
	shareRatioCheckButton(_("Disable"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	Glib::ustring tmpString;
		
	Gtk::Table *optionsTable = Gtk::manage(new Gtk::Table(5, 3, false));
	
	tmpString = "<big><big>" + Glib::ustring(_("Torrent Options")) + "</big></big>";
	
	headerLabelText.set_label(tmpString);
	headerLabelText.set_use_markup();
	headerLabelText.set_alignment(0.35, 0.5);
	headerText.add(headerLabelText);
	
	maxUploadButton.set_increments(1, 1);
	maxUploadButton.set_range(1, 20000);
	maxUploadButton.set_numeric();
			
	maxDownloadButton.set_increments(1, 1);
	maxDownloadButton.set_range(1, 20000);
	maxDownloadButton.set_numeric();
		
	maxConnectsButton.set_increments(1, 1);
	maxConnectsButton.set_range(2, 20000);
	maxConnectsButton.set_numeric();
		
	maxUploadSlotsButton.set_increments(1, 1);
	maxUploadSlotsButton.set_range(2, 20000);
	maxUploadSlotsButton.set_numeric();
	
	shareRatioButton.set_increments(0.1, 1);
	shareRatioButton.set_range(1, 1000);
	shareRatioButton.set_digits(1);
	shareRatioButton.set_numeric();
		
	// Max Upload
	maxUploadCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxUploadButtonSensitivity) );
	maxUploadButton.signal_value_changed().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxUploadChanged) );
	
	maxUploadLabel.property_xalign()  = 0.05;
	optionsTable->attach(maxUploadLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxUploadButton, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxUploadCheckButton, 2, 3, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	
	// Max Download
	maxDownloadCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxDownloadButtonSensitivity) );
	maxDownloadButton.signal_value_changed().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxDownloadChanged) );
	
	maxDownloadLabel.property_xalign()  = 0.05;
	optionsTable->attach(maxDownloadLabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxDownloadButton, 1, 2, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxDownloadCheckButton, 2, 3, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	
	// Max Connects
	maxConnectsCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxConnectsButtonSensitivity) );
	maxConnectsButton.signal_value_changed().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxConnectsChanged) );
	
	maxConnectsLabel.property_xalign()  = 0.05;
	optionsTable->attach(maxConnectsLabel, 0, 1, 2, 3, Gtk::FILL, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxConnectsButton, 1, 2, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxConnectsCheckButton, 2, 3, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	
	// Max Uploads
	maxUploadSlotsCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxUploadSlotsButtonSensitivity) );
	maxUploadSlotsButton.signal_value_changed().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setMaxUploadsSlotsChanged) );
	
	maxUploadSlotsLabel.property_xalign()  = 0.05;
	optionsTable->attach(maxUploadSlotsLabel, 0, 1, 3, 4, Gtk::FILL, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxUploadSlotsButton, 1, 2, 3, 4, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	optionsTable->attach(maxUploadSlotsCheckButton, 2, 3, 3, 4, Gtk::SHRINK, Gtk::SHRINK, 5, 5);
	
	// Share Ratio
	shareRatioCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setShareRatioButtonSensitivity) );
	shareRatioButton.signal_value_changed().connect( sigc::mem_fun(*this, &OptionsInfoNoteBookTab::setShareRatioChanged) );
	
	shareRatioHBox.pack_start(shareRatioLabel1, Gtk::PACK_SHRINK , 5);
	shareRatioHBox.pack_start(shareRatioLabel2, Gtk::PACK_SHRINK , 5);
	shareRatioHBox.pack_start(shareRatioButton, Gtk::PACK_SHRINK , 5);
	shareRatioHBox.pack_start(shareRatioLabel3, Gtk::PACK_SHRINK , 5);
	shareRatioHBox.pack_start(shareRatioCheckButton, Gtk::PACK_SHRINK , 5);
	
	maxUploadCheckButton.set_active();
	maxDownloadCheckButton.set_active();
	maxConnectsCheckButton.set_active();
	maxUploadSlotsCheckButton.set_active();
	shareRatioCheckButton.set_active();
	
	vbox.pack_start(headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*optionsTable, Gtk::PACK_SHRINK , 0);
	vbox.pack_start(shareRatioHBox, Gtk::PACK_SHRINK , 5);
	add(vbox);
	
	maxUploadChanged = maxDownloadChanged = false;
	maxConnectsChanged = maxUploadSlotsChanged = false;
	areOptionsInitialized = shareRatioChanged = false;
	
	std::list<Gtk::Widget *> focus_order;
	vbox.set_focus_chain(focus_order);
}

void OptionsInfoNoteBookTab::updateOptionsInfoNoteBookTab(Torrent *t){
	if (!areOptionsInitialized){
		areOptionsInitialized = true;
		if (t->max_upload_speed() != -1)
		{
			maxUploadCheckButton.set_active(false);
			maxUploadButton.set_value(t->max_upload_speed());
		}
		
		if (t->max_download_speed() != -1)
		{
			maxDownloadCheckButton.set_active(false);
			maxDownloadButton.set_value(t->max_download_speed());
		}
		
		if (t->max_connects() != -1)
		{
			maxConnectsCheckButton.set_active(false);
			maxConnectsButton.set_value(t->max_connects());
		}
		
		if (t->upload_slots() != -1)
		{
			maxUploadSlotsCheckButton.set_active(false);
			maxUploadSlotsButton.set_value(t->upload_slots());
		}
		
		if (t->share_ratio() != 0)
		{
			shareRatioCheckButton.set_active(false);
			shareRatioButton.set_value(t->share_ratio());
		}
			
	}
	
	int max_upload_speed = maxUploadButton.get_value_as_int();
	if (maxUploadChanged || (t->max_upload_speed() != max_upload_speed) )
	{
		if (t->max_upload_speed() != max_upload_speed && !maxUploadChanged)
		{
			if (t->max_upload_speed() != -1)
				maxUploadCheckButton.set_active(false);
			else
				maxUploadCheckButton.set_active(true);
			
			maxUploadButton.set_value(t->max_upload_speed());
		}
		else
		{
			maxUploadChanged = false;
			if(maxUploadCheckButton.get_active())
				t->set_max_upload_speed(-1);
			else
				t->set_max_upload_speed(max_upload_speed);	
		}
	}
	
	int max_download_speed = maxDownloadButton.get_value_as_int();
	if (maxDownloadChanged || (t->max_download_speed() != max_download_speed))
	{
		if (t->max_download_speed() != max_download_speed && !maxDownloadChanged)
		{
			if (t->max_download_speed() != -1)
				maxDownloadCheckButton.set_active(false);
			else
				maxDownloadCheckButton.set_active(true);
			
			maxDownloadButton.set_value(t->max_download_speed());
		}
		else
		{
			maxDownloadChanged = false;
			if (maxDownloadCheckButton.get_active())
				t->set_max_download_speed(-1);
			else
				t->set_max_download_speed(max_download_speed);
		}
	}
	
	int max_connects = maxConnectsButton.get_value_as_int();
	if (maxConnectsChanged || (t->max_connects() != max_connects))
	{
		if (t->max_connects() != max_connects && !maxConnectsChanged)
		{
			if (t->max_connects() != -1)
				maxConnectsCheckButton.set_active(false);
			else
				maxConnectsCheckButton.set_active(true);
			
			maxConnectsButton.set_value(t->max_connects());
		}
		else
		{
			maxConnectsChanged = false;
			if (maxConnectsCheckButton.get_active())
				t->set_max_connects(-1);
			else
				t->set_max_connects(max_connects);
		}
	}
	
	int max_uploads = maxUploadSlotsButton.get_value_as_int();
	if (maxUploadSlotsChanged || (t->upload_slots() != max_uploads))
	{
		if (t->upload_slots() != max_uploads && !maxUploadSlotsChanged)
		{
			if (t->upload_slots() != -1)
				maxUploadSlotsCheckButton.set_active(false);
			else
				maxUploadSlotsCheckButton.set_active(true);
			
			maxUploadSlotsButton.set_value(t->upload_slots());
		}
		else{
			maxUploadSlotsChanged = false;
			if (maxUploadSlotsCheckButton.get_active())
				t->set_upload_slots(-1);
			else
				t->set_upload_slots(max_uploads);
		}
	}
	
	float share_ratio = static_cast<float>(shareRatioButton.get_value());
	if (shareRatioChanged || (t->share_ratio() != share_ratio) )
	{
		if (t->share_ratio() != share_ratio && !shareRatioChanged)
		{
			if (t->share_ratio() != 0)
				shareRatioCheckButton.set_active(false);
			else
				shareRatioCheckButton.set_active(true);
			
			shareRatioButton.set_value(t->share_ratio());
		}
		else
		{
			shareRatioChanged = false;
			maxUploadSlotsChanged = false;
			if (shareRatioCheckButton.get_active())
				t->set_share_ratio(0.0);
			else
				t->set_share_ratio(share_ratio);
		}
	}
}

void OptionsInfoNoteBookTab::setMaxUploadButtonSensitivity(){
	maxUploadChanged = true;
	
	if (maxUploadCheckButton.get_active())
		maxUploadButton.property_sensitive () = false;
	else
		maxUploadButton.property_sensitive () = true;
}

void OptionsInfoNoteBookTab::setMaxDownloadButtonSensitivity(){
	maxDownloadChanged = true;
	
	if (maxDownloadCheckButton.get_active())
		maxDownloadButton.property_sensitive () = false;
	else
		maxDownloadButton.property_sensitive () = true;
}

void OptionsInfoNoteBookTab::setMaxConnectsButtonSensitivity(){
	maxConnectsChanged = true;
	
	if (maxConnectsCheckButton.get_active())
		maxConnectsButton.property_sensitive () = false;
	else
		maxConnectsButton.property_sensitive () = true;
}

void OptionsInfoNoteBookTab::setMaxUploadSlotsButtonSensitivity(){
	maxUploadSlotsChanged = true;
	
	if (maxUploadSlotsCheckButton.get_active())
		maxUploadSlotsButton.property_sensitive () = false;
	else
		maxUploadSlotsButton.property_sensitive () = true;
}

void OptionsInfoNoteBookTab::setShareRatioButtonSensitivity(){
	shareRatioChanged = true;
	
	if (shareRatioCheckButton.get_active()){
		shareRatioButton.property_sensitive () = false;
		shareRatioLabel2.property_sensitive () = false;
		shareRatioLabel3.property_sensitive () = false;
	}
	else{
		shareRatioButton.property_sensitive () = true;
		shareRatioLabel2.property_sensitive () = true;
		shareRatioLabel3.property_sensitive () = true;
	}
}

void OptionsInfoNoteBookTab::setMaxUploadChanged(){
	maxUploadChanged = true;
}

void OptionsInfoNoteBookTab::setMaxDownloadChanged(){
	maxDownloadChanged = true;
}

void OptionsInfoNoteBookTab::setMaxConnectsChanged(){
	maxConnectsChanged = true;
}

void OptionsInfoNoteBookTab::setMaxUploadsSlotsChanged(){
	maxUploadSlotsChanged = true;
}

void OptionsInfoNoteBookTab::setShareRatioChanged(){
	shareRatioChanged = true;
}

OptionsInfoNoteBookTab::~OptionsInfoNoteBookTab(){
}
